==> eslint-plugin-mocha-4.12.0.tgz: package/lib/rules/no-setup-in-describe.js <==
'use strict';

var astUtils = require('../util/ast'),
    additionalSuiteNames = require('../util/settings').additionalSuiteNames;

module.exports = function noSetupInDescribe(context) {
    var nesting = [],
        settings = context.settings,
        FUNCTION = 1,
        DESCRIBE = 2,
        // "Pure" nodes are hooks (like `beforeEach`) or `it` calls
        PURE = 3;

    function isPureNode(node) {
        return astUtils.isHookIdentifier(node) || astUtils.isTestCase(node);
    }

    function reportCallExpression(callExpression) {
        var message = 'Unexpected function call in describe block.';

        context.report({
            message: message,
            node: callExpression.callee
        });
    }

    function reportMemberExpression(memberExpression) {
        var message = 'Unexpected member expression in describe block. ' +
            'Member expressions may call functions via getters.';

        context.report({
            message: message,
            node: memberExpression
        });
    }

    function isNestedInDescribeBlock() {
        return nesting.length &&
                nesting.indexOf(PURE) === -1 &&
                nesting.lastIndexOf(FUNCTION) < nesting.lastIndexOf(DESCRIBE);
    }

    function handleCallExpressionInDescribe(node) {
        if (isPureNode(node)) {
            nesting.push(PURE);
        } else if (isNestedInDescribeBlock()) {
            reportCallExpression(node);
        }
    }

    return {
        CallExpression: function (node) {
            var isDescribe = astUtils.isDescribe(node, additionalSuiteNames(settings));
            if (isDescribe) {
                nesting.push(DESCRIBE);
                return;
            }
            // don't process anything else if the first describe hasn't been processed
            if (!nesting.length) {
                return;
            }
            handleCallExpressionInDescribe(node);
        },

        'CallExpression:exit': function (node) {
            if (astUtils.isDescribe(node) || nesting.length && isPureNode(node)) {
                nesting.pop();
            }
        },

        MemberExpression: function (node) {
            if (isNestedInDescribeBlock()) {
                reportMemberExpression(node);
            }
        },

        FunctionDeclaration: function () {
            if (nesting.length) {
                nesting.push(FUNCTION);
            }
        },
        'FunctionDeclaration:exit': function () {
            if (nesting.length) {
                nesting.pop();
            }
        },

        ArrowFunctionExpression: function () {
            if (nesting.length) {
                nesting.push(FUNCTION);
            }
        },
        'ArrowFunctionExpression:exit': function () {
            if (nesting.length) {
                nesting.pop();
            }
        }
    };
};

==> eslint-plugin-mocha-5.0.0.tgz: package/lib/rules/no-setup-in-describe.js <==
'use strict';

const astUtils = require('../util/ast');
const { additionalSuiteNames } = require('../util/settings');

const FUNCTION = 1;
const DESCRIBE = 2;
// "Pure" nodes are hooks (like `beforeEach`) or `it` calls
const PURE = 3;

module.exports = function noSetupInDescribe(context) {
    const nesting = [];
    const settings = context.settings;

    function isPureNode(node) {
        return astUtils.isHookIdentifier(node) || astUtils.isTestCase(node);
    }

    function reportCallExpression(callExpression) {
        const message = 'Unexpected function call in describe block.';

        context.report({
            message,
            node: callExpression.callee
        });
    }

    function reportMemberExpression(memberExpression) {
        const message = 'Unexpected member expression in describe block. ' +
            'Member expressions may call functions via getters.';

        context.report({
            message,
            node: memberExpression
        });
    }

    function isNestedInDescribeBlock() {
        return nesting.length &&
                nesting.indexOf(PURE) === -1 &&
                nesting.lastIndexOf(FUNCTION) < nesting.lastIndexOf(DESCRIBE);
    }

    function handleCallExpressionInDescribe(node) {
        if (isPureNode(node)) {
            nesting.push(PURE);
        } else if (isNestedInDescribeBlock()) {
            reportCallExpression(node);
        }
    }

    return {
        CallExpression(node) {
            const isDescribe = astUtils.isDescribe(node, additionalSuiteNames(settings));
            if (isDescribe) {
                nesting.push(DESCRIBE);
                return;
            }
            // don't process anything else if the first describe hasn't been processed
            if (!nesting.length) {
                return;
            }
            handleCallExpressionInDescribe(node);
        },

        'CallExpression:exit'(node) {
            if (astUtils.isDescribe(node) || nesting.length && isPureNode(node)) {
                nesting.pop();
            }
        },

        MemberExpression(node) {
            if (isNestedInDescribeBlock()) {
                reportMemberExpression(node);
            }
        },

        FunctionDeclaration() {
            if (nesting.length) {
                nesting.push(FUNCTION);
            }
        },
        'FunctionDeclaration:exit'() {
            if (nesting.length) {
                nesting.pop();
            }
        },

        ArrowFunctionExpression() {
            if (nesting.length) {
                nesting.push(FUNCTION);
            }
        },
        'ArrowFunctionExpression:exit'() {
            if (nesting.length) {
                nesting.pop();
            }
        }
    };
};
